/******************************************************************************
  BMP2HP.PRG - Functions to print BMP file to HP-PCL5 printer
  Written by:  Alain Boucher, July 1997.
  CopyRight:   ACTI-$OFT(r) - 2329 6684 QUEBEC INC.
               1363, Des Peupliers
               Mont St-Hilaire, Qc.
               Canada  J3G 4S6
               Tel: (514)944-4949
               Fax: (450)464-5726
               Web: http://www.acti-soft.com
               E-Mail: aboucher@acti-soft.com
******************************************************************************/
//    COMPILE: CLIPPER BMP2HP /M/N
//  IMPORTANT: Needs CT.LIB or CTP.LIB for these two functions only:
//                                         CTON()
//                                         NTOC()
*******************************************************************************
/* Fonction BmpRead(), lit un fichier BMP monochrome et le place dans le array
   aPixels pass en rfrence.
   Les paramtres sont:
       cBmpFiche  C  Nom du fichier graphique
       aPixels    A  Array qui contiendra les pixels du graphique
       lSilent    L  Affichage ou non  l'cran du travail en cours
       nRow       N  No de la ligne d'affichage
       nCol       N  No de la colonne d'affichage
   La variable numrique nOk est le code de vrification, soit:
       0 - Succs
       1 - Nom de fichier incorrect
       2 - Fichier n'existe pas
       3 - Type de fichier incorrect
       4 - Fichier n'est pas monochrome
       5 - Image est compresse
       6 - Erreur de lecture
       7 - Lecture incomplte
      99 - Annuler par l'usager.                                    */

Func BmpRead( cBmpFiche,aPixels )

Local OldScr,OldCol,OldCur,aErreurs,cBoite,;
      nOk,FilH,cV,nV,nLigne,nColone,cRead,nBmpType,;
      nOffset,nHeaderLen,nHeight,nIWidth,nBWidth,;
      cBmpPixels,cHpPixels,nChrono,nRow,nCol,lSilent

nOffSet   := nHeaderLen := nHeight := nWidth := nV := Cnt := FilH := nOk := ;
            nLigne := nColone := nBmpType := 0
cV        := ''
lSilent   := if( lSilent == NIL,.f.,lSilent )
nRow      := if( nRow == NIL,12,nRow )
nCol      := if( nCol == NIL,5,nCol )
cBoite    := '͸Գ '

aErreurs  := { "Incorrect file name", ;
               "File not found", ;
               "Incorrect file type", ;
               "Not monochrome or more than 16 colors", ;
               "Image data is compressed", ;
               "File open error", ;
               "Read error", ;
               "Incomplete file read", ;
               "File too big to fit in memory" }

cBmpFiche := Upper( Alltrim( cBmpFiche ) )

if !lSilent
  OldScr := SaveScreen( nRow,nCol,nRow+7,nCol+70 )
  OldCol := SetColor( "N/W,N/W" )
  OldCur := SetCursor( 0 )
  @ nRow,nCol,nRow+7,nCol+70 Box cBoite
  SetColor( "W+/B,W+/B" )
  @ nRow,nCol Say Space(71)
  @ nRow,nCol+2 Say 'BmpRead() by ACTI-$OFT(r)    FOR DEMO ONLY'
  SetColor( "N/W,N/W" )
  @ nRow+2,nCol+2 say 'File name:'
  @ nRow+2,nCol+15 Say Alltrim( Upper( cBmpFiche ) )
End

if At( ' ',cBmpFiche ) <> 0
  nOk := 1
End

if !File( cBmpFiche )
  nOk := 2
End

if nOk <> 0
  if !lSilent
    @ nRow+4,nCol+35 - ( Len( aErreurs[nOk] ) / 2 ) Say aErreurs[nOk]
    Tone( 500,3 )
    SetColor( "R+/W,R+/W" )
    @ nRow+7,nCol+2 Say 'Hit [Esc] to continue...'
    Clear TypeaHead
    While Inkey(0) <> 27
    End
    RestScreen( nRow,nCol,nRow+7,nCol+70,OldScr )
    SetCOlor( OldCOl )
    SetCursor( OldCur )
  End
  Return( nOk )
End

FilH := FOpen( cBmpFiche )
if Ferror() <> 0
  nOk := 6
  FClose( FilH )
  if !lSilent
    @ nRow+4,nCol+35 - ( Len( aErreurs[nOk] ) / 2 ) Say aErreurs[nOk]
    Tone( 500,3 )
    SetColor( "R+/W,R+/W" )
    @ nRow+7,nCol+2 Say 'Hit [Esc] to continue...'
    Clear TypeaHead
    While Inkey(0) <> 27
    End
    RestScreen( nRow,nCol,nRow+7,nCol+70,OldScr )
    SetCOlor( OldCOl )
    SetCursor( OldCur )
  End
  Return( nOk )
End

cV := FReadStr( FilH,2 )
if !cV == 'BM'
  nOk := 3
  FClose( FilH )
  if !lSilent
    @ nRow+4,nCol+35 - ( Len( aErreurs[nOk] ) / 2 ) Say aErreurs[nOk]
    Tone( 500,3 )
    SetColor( "R+/W,R+/W" )
    @ nRow+7,nCol+2 Say 'Hit [Esc] to continue...'
    Clear TypeaHead
    While Inkey(0) <> 27
    End
    RestScreen( nRow,nCol,nRow+7,nCol+70,OldScr )
    SetCOlor( OldCOl )
    SetCursor( OldCur )
  End
  Return( nOk )
End
cV         := Space(4)
FRead( FilH,@cV,4 )
nV         := Bin2L( cV )                          // Taille du fichier

FSeek( FilH,4,1 )                                  // Toujours "00x00x00x00x"

cV         := Space(4)
FRead( FilH,@cV,4 )
nOffSet    := Bin2L( cV )                          // OffSet du header au data

cV         := Space(2)
FRead( FilH,@cV,2 )
cV         := cV + Chr(0) + Chr(0)
nHeaderLen := Bin2L( cV )                          // Taille du header
FSeek( FilH,2,1 )                                  // Toujours "00x00x"

cV         := Space(4)
FRead( FilH,@cV,4 )
nIWidth    := Bin2L( cV )                          // Largeur de l'image
cV         := Space(4)
FRead( FilH,@cV,4 )
nHeight    := Bin2L( cV )                          // Hauteur de l'image
if !lSilent
  @ nRow+3,nCol+2 say 'Image:'
  @ nRow+3,nCol+15 say Alltrim( Transform( nHeight,'9999999999' ) ) + "H x " + ;
                       Alltrim( Transform( nIWidth,'9999999999' ) ) + "W"
End
cV         := Space(2)
FRead( FilH,@cV,2 )
cV         := cV + Chr(0) + Chr(0)
nV         := Bin2L( cV )                          // Nombre de planes

if nV <> 1                                         // Toujours "1"
  nOk := 4
  FClose( FilH )
  if !lSilent
    @ nRow+4,nCol+35 - ( Len( aErreurs[nOk] ) / 2 ) Say aErreurs[nOk]
    Tone( 500,3 )
    SetColor( "R+/W,R+/W" )
    @ nRow+7,nCol+2 Say 'Hit [Esc] to continue...'
    Clear TypeaHead
    While Inkey(0) <> 27
    End
    RestScreen( nRow,nCol,nRow+7,nCol+70,OldScr )
    SetCOlor( OldCOl )
    SetCursor( OldCur )
  End
  Return( nOk )
End

cV         := Space(2)
FRead( FilH,@cV,2 )
cV         := cV + Chr(0) + Chr(0)
nBmpType   := Bin2L( cV )                          // Bits par pixels
if nBmpType <> 1 .and. nBmpType <> 4               // "1" est monochrome
  FClose( FilH )                                   // "4" est 16 couleurs
  nOk := 4
  if !lSilent
    @ nRow+4,nCol+35 - ( Len( aErreurs[nOk] ) / 2 ) Say aErreurs[nOk]
    Tone( 500,3 )
    SetColor( "R+/W,R+/W" )
    @ nRow+7,nCol+2 Say 'Hit [Esc] to continue...'
    Clear TypeaHead
    While Inkey(0) <> 27
    End
    RestScreen( nRow,nCol,nRow+7,nCol+70,OldScr )
    SetCOlor( OldCOl )
    SetCursor( OldCur )
  End
  Return( nOk )
End

if nIWidth * nBmpType > 2000                       // Longueur maximum d'un
  FClose( FilH )                                   // string est 256 carac.
  nOk := 9
  if !lSilent
    @ nRow+4,nCol+35 - ( Len( aErreurs[nOk] ) / 2 ) Say aErreurs[nOk]
    Tone( 500,3 )
    SetColor( "R+/W,R+/W" )
    @ nRow+7,nCol+2 Say 'Hit [Esc] to continue...'
    Clear TypeaHead
    While Inkey(0) <> 27
    End
    RestScreen( nRow,nCol,nRow+7,nCol+70,OldScr )
    SetCOlor( OldCOl )
    SetCursor( OldCur )
  End
  Return( nOk )
End

if !lSilent
  @ nRow+3,nCol+35 Say 'File type:'
  @ nRow+3,nCol+50 Say if( nBmpType==1,'Mono','16 colors' ) + ' (' + ;
                   Alltrim( Transform( nV,'999,999,999' ) ) + ' bytes)'
End
cV         := Space(2)
FRead( FilH,@cV,2 )
cV         := cV + Chr(0) + Chr(0)
nV         := Bin2L( cV )                          // Type de compression
if nV <> 0                                         // "0" est aucun
  FClose( FilH )
  nOk := 5
  if !lSilent
    @ nRow+4,nCol+35 - ( Len( aErreurs[nOk] ) / 2 ) Say aErreurs[nOk]
    Tone( 500,3 )
    SetColor( "R+/W,R+/W" )
    @ nRow+7,nCol+2 Say 'Hit [Esc] to continue...'
    Clear TypeaHead
    While Inkey(0) <> 27
    End
    RestScreen( nRow,nCol,nRow+7,nCol+70,OldScr )
    SetCOlor( OldCOl )
    SetCursor( OldCur )
  End
  Return( nOk )
End

cV         := Space(4)
FRead( FilH,@cV,4 )
nV         := Bin2L( cV )                          // Taille de l'image
FSeek( FilH,4,1 )                                  // Toujours "00x00x00x00x"

cV         := Space(4)
FRead( FilH,@cV,4 )
nV         := Bin2L( cV )                          // pixels X par mtre
cV         := Space(4)
FRead( FilH,@cV,4 )
nV         := Bin2L( cV )                          // pixels Y par mtre
cV         := Space(4)
FRead( FilH,@cV,4 )
nV         := Bin2L( cV )                          // Couleurs utilises
cV         := Space(4)
FRead( FilH,@cV,4 )
nV         := Bin2L( cV )                          // Couleurs importantes

// Positionne au dbut du data
Fseek( FilH,nOffSet,0 )

aPixels := {}
cV      := ''

nBWidth := nIWidth
nV      := nIWidth / if( nBmpType==1,32,16 )
if !Int( nV ) == nV
  nBWidth := ( Int( nV ) + 1 ) * if( nBmpType==1,32,16 )
End

nChrono := Seconds()
nV      := 0

if !lSilent
  @ nRow+4,nCol+2 Say 'Reading:'
End

For nLigne = 1 to nHeight

  if !lSilent
    ++nV
    @ nRow+4,nCol+15 Say Transform( ( nV * 100 ) / nHeight,'999.99' ) + '%'
  End
  cBmpPixels := Space( nBWidth / if( nBmpType==1,8,2 ) )
  FRead( FilH,@cBmpPixels,nBWidth/if( nBmpType==1,8,2 ) )
  if nBmpType == 1
    cHpPixels := BmpMono2Hp( cBmpPixels,nIWidth )
    Else
      cHpPixels := Bmp16C2Hp( cBmpPixels,nIWidth )
  End

  aAdd( aPixels,cHpPixels )

Next
aAdd( aPixels,nBmpType )

nChrono := Seconds() - nChrono

FClose( fIlH )

if !lSilent
  @ nRow+4,nCol+35 Say "Result:"
  @ nRow+4,nCol+50 Say if( nOk==99,'Aborted ','Success ' ) + Transform( nChrono,'999999.999' )
  Tone( 3000,1 )
  SetColor( "R+/W,R+/W" )
  @ nRow+7,nCol+2 Say 'Hit [Esc] to continue...'
  Clear TypeaHead
  While Inkey(0) <> 27
  End
  RestScreen( nRow,nCol,nRow+7,nCol+70,OldScr )
  SetCOlor( OldCOl )
  SetCursor( OldCur )
End

Return( nOk )
*******************************************************************************
Stat Func BmpMono2Hp( cBmpLine,nIWidth )

Local cBmpBin,cHpBin,cHpPixel,cV,Cnt,nV,nBit2Keep,nByte,cBmpPixel,cHpLine

cHpLine := ''
For nByte = 1 to Len( cBmpLine )
  cBmpPixel := SubStr( cBmpLine,nByte,1 )
  nBit2Keep := 8
  nV        := nByte * 8
  if nV > nIWidth
    nBit2Keep := 8 - ( nV - nIWidth )
    if nBit2Keep == 0
      cHpLine := cHpLine + Chr( 0 )
      Loop
    End
  End
  cBmpBin := NTOC( Asc( cBmpPixel ),2,8,"0" )

  cHpBin  := StrTran( cBmpBin,'0','X' )
  cHpBin  := StrTran( cHpBin,'1','0' )
  cHpBin  := StrTran( cHpBin,'X','1' )
  if nBit2Keep < 8
    cHpBin := Left( cHpBin,nBit2Keep ) + Repl( '0',8-nBit2Keep )
  End
  cHpLine  := cHpLine + Chr( CTON( cHpBin,2 ) )

Next

Return( cHpLine )
*******************************************************************************
Stat Func Bmp16C2Hp( cBmpLine,nIWidth )

Local cV,nV,cHex

if Len( cBmpLine ) > nIWidth /2
  nV := Int( nIWidth / 2 )
  if ( nV * 2 ) < nIWidth
    ++nV
  End
  cBmpLine := Left( cBmpLine,nV )
  if ( nV * 2 ) > nIWidth
    cV       := Right( cBmpLine,1 )
    cBmpLine := Left( cBmpLine,Len( cBmpLine ) - 1 )
    cHex     := NTOC( Asc( cV ),16,2,'0' )
    cHex     := Left( cHex,1 ) + 'F'
    cV       := Chr( CTON( cHex,16 ) )
    cBmpLine := cBmpLine + cV
  End
End

Return( cBmpLine )
*******************************************************************************
/* Fonction BmpHpPrint() Impression d'un graphic BMP sur imprimante HP.

   Les paramtres sont:
        cBmpNom     C  Nom du fichier Graphique
        aPixels     A  Array qui contient les pixels du graphique
        nXCoord     N  Point de dpart sur l'axe horizontal (en pouce)
        nYCoord     N  Point de dpart sur l'axe vertical (en pouce)
        nXSize      N  Largeur du graphique (en pouce)
        nYSize      N  Hauteur du graphique (en pouce)
        nResolution N  Rsolution  utiliser (0 pour automatique)
        lFrame      L  Impression d'un cdre autour du graphique
        lSilent     L  Pas de message  l'cran
        nRow        N  No de la ligne d'affichage
        nCol        N  No de la colonne d'affichage

   Cette fonction retourne:
                             0 - Ok
                            -1 - Erreur
                            99 - Annuler par l'usager
*/
Func BmpHpPrint( cBmpNom,aPixels,nXCoord,nYCoord,nXSize,nYSize, ;
                 nResolution,lFrame )

Local OldScr,OldCol,OldCur,OldCon,lSilent,nRow,nCol,cBoite,;
      cEsc,Cnt,nHeight,nWidth,cXCoord,cYCoord,cResolution,cWidth, ;
      nV,nBmpType, ;
      aLigne,nLigne

cEsc        := Chr(27)
aPixels     := if( aPixels==NIL,{},aPixels )
nXCoord     := if( nXCoord==NIL,0,nXCoord )
nYCoord     := if( nYCoord==NIL,0,nYCoord )
nXSize      := if( nXSize==NIL,0,nXSize )
nYSize      := if( nYSize==NIL,0,nYSize )
nResolution := if( nResolution==NIL,300,nResolution )
lFrame      := if( lFrame==NIL,.f.,lFrame )
lSilent     := if( lSilent==NIL,.f.,lSilent )
nRow        := if( nRow==NIL,12,nRow )
nCol        := if( nCol==NIL,5,nCol )
cBoite      := '͸Գ '

OldCon := Set( _SET_CONSOLE,.f. )

if !lSilent
  OldScr := SaveScreen( nRow,nCOl,nRow+8,nCol+70 )
  OldCol := SetColor( "N/W,N/W" )
  OldCur := SetCursor( 0 )
  @ nRow,nCol,nRow+8,nCol+70 Box cBoite
  SetColor( "W+/B,W+/B" )
  @ nRow,nCol Say Space(71)
  @ nRow,nCol+2 Say 'BmpHpPrint() by ACTI-$OFT(r)     FOR DEMO ONLY'
  SetColor( "N/W,N/W" )
  @ nRow+2,nCol+2 say 'File name:'
  @ nRow+2,nCol+15 Say Alltrim( Upper( cBmpNom ) )
End

if Len( aPixels ) == 0 .or. Len( cBmpNom ) == 0
  if !lSilent
    @ nRow+5,nCol+35 say 'Status:'
    @ nRow+5,nCol+50 say 'Invalid data!!!'
    Tone( 3000,1 )
    SetColor( "R+/W,R+/W" )
    @ nRow+8,nCol+2 Say 'Hit [Esc] to continue...'
    Clear TypeaHead
    While Inkey(0) <> 27
    End
    RestScreen( nRow,nCOl,nRow+8,nCol+70,OldScr )
    SetColor( OldCol )
    SetCursor( OldCur )
  End
  Set( _SET_CONSOLE,OldCon )
  Return( -1 )
End

if !Set( _SET_PRINTER )
  if !lSilent
    @ nRow+5,nCol+35 say 'Status:'
    @ nRow+5,nCol+50 say 'Printer not set!!!'
    Tone( 3000,1 )
    SetColor( "R+/W,R+/W" )
    @ nRow+8,nCol+2 Say 'Hit [Esc] to continue...'
    Clear TypeaHead
    While Inkey(0) <> 27
    End
    RestScreen( nRow,nCOl,nRow+8,nCol+70,OldScr )
    SetColor( OldCol )
    SetCursor( OldCur )
  End
  Set( _SET_CONSOLE,OldCon )
  Return( -1 )
End

nHeight     := Len( aPixels ) - 1
nWidth      := Len( aPixels[1] ) + if( lFrame,2,0 )
nBmpType    := aPixels[ nHeight+1 ]
if !lSilent
  @ nRow+3,nCol+2 say 'Dimensions:'
  @ nRow+3,nCol+15 Say  + Alltrim( Transform( nHeight,'9999999999' ) ) + 'H x ' + ;
                          Alltrim( Transform( nWidth*8,'9999999999' ) ) + 'W'
End

if nResolution == 0 .and. nXSize == 0 .and. nYSize == 0
  nResolution := if( nBmpType==1,150,300 )
End
if nResolution == 0
  nResolution := BmpSizeFit( nWidth*8, ;
                             nHeight + if( lFrame,16,0 ), ;
                             nXSize,nYSize )
End

nResolution := if( !nResolution==75 .and. ;
                   !nResolution==100 .and. ;
                   !nResolution==150 .and. ;
                   !nResolution==300 .and. ;
                   !nResolution==600,75,nResolution )

if !lSilent
  @ nRow+3,nCol+35 say 'Size requ.:'
  @ nRow+3,nCol+50 Say Alltrim( Transform( nYSize,'9999999999' ) ) + '"H x ' + ;
                       Alltrim( Transform( nXSize,'9999999999' ) ) + '"W'
  @ nRow+4,nCol+2 say 'Resolution:'
  @ nRow+4,nCol+15 Say Alltrim( Transform( nResolution,'9999' ) ) + ' DPI'
  @ nRow+4,nCol+35 Say 'Position:'
  @ nRow+4,nCol+50 Say Alltrim( Transform( nYCoord,'99.999' ) ) + '"H x ' + ;
                       Alltrim( Transform( nXCoord,'99.999' ) ) + '"W'
End

nV          := nXCoord                         // Convertie la position X de
nV          := nV - .25                        // pouces  pixels.
nV          := if( nV < 0,0,nV )               // 1/4"  gauche de moins.
nXCoord     := Int( nV * 300 )
nV          := nYCoord                         // Convertie la position Y de
nV          := nV - 0.50                       // pouces  pixels.
nV          := if( nV < 0,0,nV )               // 1/2" en haut de moins.
nYCoord     := Int( nV * 300 )

cXCoord     := Alltrim( Transform( nXCoord,'9999999999' ) )
cYCoord     := Alltrim( Transform( nYCoord,'9999999999' ) )
cResolution := Alltrim( Transform( nResolution,'9999' ) )
cWidth      := Alltrim( Transform( nWidth,'9999999999' ) )

?? cEsc + "*p" + cXCoord + "x" + cYCoord + "Y"  // Position XY en pixel
?? cEsc + "*t" + cResolution + "R"              // Rsolution en DPI
?? cEsc + "*r0F"                                // Orientation Portrait
?? cEsc + "*r1A"                                // Dbute  la position actu-
                                                // elle du curseur
if !lSilent
  @ nRow+5,nCol+2 say 'Printing:'
End
if lFrame
  For Cnt = 1 to 8
    ?? cEsc + "*b0m" + cWidth + "W" + ;
       if( Cnt <= 2,Repl( Chr(255),nWidth ), ;
                    Chr(192) + Repl( Chr(0),nWidth-2 ) + Chr(3) )
  Next
End

nV         := 0
cOldPixels := ""
aLigne     := { "","","","" }

For Cnt = nHeight to 1 step -1

  if !lSilent
    ++nV
    @ nRow+5,nCol+15 Say Transform( ( nV * 100 ) / nHeight,'999.99' ) + '%'
  End

  if nBmpType == 1
    ?? cEsc + "*b0m" + cWidth + "W" + ;
       if( lFrame,Chr(192),'' ) + aPixels[ Cnt ] + if( lFrame,Chr(3),'' )
    Else
      if !aPixels[Cnt] == cOldPixels
        Hp4BitShades( aPixels[Cnt],@aLigne )
      End
      cOldPixels := aPixels[Cnt]
      For nLigne = 1 to 4
        cWidth := Alltrim( Transform( Len( aLigne[nLigne] ) + ;
                                      if( lFrame,2,0 ),'999999' ) )
        ?? cEsc + "*b0m" + cWidth + "W" + ;
           if( lFrame,Chr(192),'' ) + aLigne[nLigne] + if( lFrame,Chr(3),'' )
      Next
  End
Next

if lFrame
  For Cnt = 1 to 8
    ?? cEsc + "*b0m" + cWidth + "W" + ;
       if( Cnt > 6,Repl( Chr(255),nWidth ), ;
                   Chr(192) + Repl( Chr(0),nWidth-2 ) + Chr(3) )
  Next
End

?? cEsc + "*rB"                              // Fin du raster graphic

if !lSilent
  @ nRow+5,nCol+35 say 'Status:'
  @ nRow+5,nCol+50 say 'Completed'
  Tone( 3000,1 )
  SetColor( "R+/W,R+/W" )
  @ nRow+8,nCol+2 Say 'Hit [Esc] to continue...'
  Clear TypeaHead
  While Inkey(0) <> 27
  End
  RestScreen( nRow,nCOl,nRow+8,nCol+70,OldScr )
  SetColor( OldCol )
  SetCursor( OldCur )
End

Set( _SET_CONSOLE,OldCon )

Return( 0 )
*****************************************************************************
Stat Func Hp4BitShades( cLigne,aLigne )

Local nV,Cnt,cV,nLen,cHex,nLigne
Static aShades

if aShades == NIL
  aShades := { { "F","F","F","F" }, ;  // 0
               { "F","E","7","F" }, ;  // 1
               { "F","E","7","B" }, ;  // 2
               { "D","E","7","B" }, ;  // 3
               { "D","E","6","B" }, ;  // 4
               { "D","6","6","B" }, ;  // 5
               { "D","6","6","9" }, ;  // 6
               { "9","6","6","9" }, ;  // 7
               { "9","4","6","9" }, ;  // 8
               { "9","4","2","9" }, ;  // 9
               { "9","4","0","9" }, ;  // A
               { "9","0","0","9" }, ;  // B
               { "9","0","0","1" }, ;  // C
               { "8","0","0","1" }, ;  // D
               { "8","0","0","0" }, ;  // E
               { "0","0","0","0" } }   // F
End

nLen   := Len( cLigne )
aLigne := { '','','','' }

For Cnt = 1 to nLen
  cHex      := NTOC( Asc( SubStr( cLigne,Cnt,1 ) ),16,2,"0" )
  For nLigne = 1 to 4
    cV := aShades[ ( CTON( Left( cHex,1 ),16 ) + 1 ) ][nLigne] + ;
          aShades[ ( CTON( Right( cHex,1 ),16 ) + 1 ) ][nLigne]
    aLigne[ nLigne ] := aLigne[ nLigne ] + Chr( CTON( cV,16 ) )
  Next
Next

Return( NIL )
*****************************************************************************
/* Fonction BmpSizeFit()  recalcule la rsolution pour que le graphique tienne
                          dans l'espace aloue par les coordonnes.
   Les paramtres sont:
        nWidth   N  Largeur du graphique en pixels
        nHeight  N  Hauteur du graphique en pixels
        nXSize   N  Largeur alloue en pouces
        nYSize   N  Hauteur alloue en pouces
*/
Stat Func BmpSizeFit( nWidth,nHeight,nXSize,nYSize )

nResolution := 75
if nWidth / nResolution <= nXSize .and. ;
   nHeight / nResolution <= nYSize
  Return( nResolution )
End

nResolution := 100
if nWidth / nResolution <= nXSize .and. ;
   nHeight / nResolution <= nYSize
  Return( nResolution )
End

nResolution := 150
if nWidth / nResolution <= nXSize .and. ;
   nHeight / nResolution <= nYSize
  Return( nResolution )
End

nResolution := 300
if nWidth / nResolution <= nXSize .and. ;
   nHeight / nResolution <= nYSize
  Return( nResolution )
End

nResolution := 600

Return( nResolution )
*******************************************************************************
Func HpReset()

Local OldCon

if !Set( _SET_PRINTER )
  Return( -1 )
End
OldCon := Set( _SET_CONSOLE,.f. )

?? Chr(27) + 'E'       // Reset et feed la page si data prsent
?? Chr(27) + "*v0N"    // Caractres transparents

Set( _SET_CONSOLE,OldCon )

Return( 0 )
*******************************************************************************
/* Fonction HpFrame()  Impression d'un cadre selon les paramtres.
   Cette fonction retourne toujours NIL.
   Les paramtres sont:
        nLeft    N  Coordon horizontale du coin suprieur gauche (en pouce)
        nTop     N  Coordon verticale du coin suprieur gauche (en pouce)
        nWidth   N  Largeur du cadre (en pouce)
        nHeight  N  Hauteur du cadre (en pouce)
*/

Func HpFrame( nLeft,nTop,nWidth,nHeight )

Local cEsc,cTop,cLeft,Cnt,cBoite,OldCon,OldScr,OldCol,OldCur

if !Set( _SET_PRINTER )
  Return( -1 )
End
OldCon := Set( _SET_CONSOLE,.f. )
OldScr := SaveScreen( 12,5,16,75 )
OldCol := SetColor( "N/W,N/W" )
OldCur := SetCursor( 0 )
cBoite := '͸Գ '
@ 12,5,16,75 Box cBoite
SetColor( "W+/B,W+/B" )
@ 12,5 Say Space(71)
@ 12,7 Say 'HpFrame() by ACTI-$OFT(r)     FOR DEMO ONLY'
SetColor( "N/W,N/W" )
@ 14,7 say 'Printing Frame...'

cEsc    := Chr(27)

nTop    := nTop - 0.50
nTop    := if( nTop < 0,0,nTop )
nTop    := Int( nTop * 300 )
nLeft   := nLeft - 0.25
nLeft   := if( nLeft < 0,0,nLeft )
nLeft   := Int( nLeft * 300 )
nHeight := nHeight * 75
nWidth  := Int( ( nWidth * 75 ) / 8 )

cTop  := Alltrim( Transform( nTop,'9999999999' ) )
cLeft := Alltrim( Transform( nLeft,'9999999999' ) )

?? cEsc + "*p" + cLeft + "x" + cTop + "Y"
?? cEsc + "*t75R"
?? cEsc + "*r0F"
?? cEsc + "*r1A"

?? cEsc + "*b1m2W" + Chr(nWidth-1) + chr(255)
for Cnt = 1 to ( nHeight - 2 )

  ?? cEsc + "*b1m6W" + Chr(0) + chr(128) + ;
                       Chr(nWidth-3) + Chr(0) + ;
                       Chr(0) + Chr(1)

Next
?? cEsc + "*b1m2W" + Chr(nWidth-1) + chr(255)
?? cEsc + "*rB"

@ 14,col()+2 say 'Completed'
Tone( 3000,1 )
SetColor( "R+/W,R+/W" )
@ 16,7 Say 'Hit [Esc] to continue...'
Clear TypeaHead
While Inkey(0) <> 27
End
RestScreen( 12,5,16,75,OldScr )
SetColor( OldCol )
SetCursor( OldCur )

Set( _SET_CONSOLE,OldCon )

Return( 0 )
*******************************************************************************
/* Fonction HpShade()  Impression d'un cadre ombrag.
   Cette fonction retourne toujours NIL.
   Les paramtres sont:
        nLeft    N  Coordon horizontale du coin suprieur gauche (en pouce)
        nTop     N  Coordon verticale du coin suprieur gauche (en pouce)
        nWidth   N  Largeur du cadre (en pouce)
        nHeight  N  Hauteur du cadre (en pouce)
*/

Func HpShade( nLeft,nTop,nWidth,nHeight )

Local cEsc,cTop,cLeft,Cnt,nWide,cWide,nReste,nV,cBoite,OldCon,OldScr,OldCol,OldCur

if !Set( _SET_PRINTER )
  Return( -1 )
End
OldCon := Set( _SET_CONSOLE,.f. )
OldScr := SaveScreen( 12,5,16,75 )
OldCol := SetColor( "N/W,N/W" )
OldCur := SetCursor( 0 )
cBoite := '͸Գ '
@ 12,5,16,75 Box cBoite
SetColor( "W+/B,W+/B" )
@ 12,5 Say Space(71)
@ 12,7 Say 'HpShade() by ACTI-$OFT(r)     FOR DEMO ONLY'
SetColor( "N/W,N/W" )
@ 14,7 say 'Printing Shade...'

cEsc    := Chr(27)
nWide   := 0
nReste  := 0

nTop    := nTop - 0.50
nTop    := if( nTop < 0,0,nTop )
nTop    := Int( nTop * 300 )
nLeft   := nLeft - 0.25
nLeft   := if( nLeft < 0,0,nLeft )
nLeft   := Int( nLeft * 300 )
nHeight := nHeight * 600
nWidth  := Int( ( nWidth * 600 ) / 8 )

if nWidth > 255
  Cnt := nWidth
  While .t.
    Cnt := Cnt - 255
    if Cnt < 0
      nReste := Cnt + 255
      Exit
    End
    ++nWide
  End
End

cTop  := Alltrim( Transform( nTop,'9999999999' ) )
cLeft := Alltrim( Transform( nLeft,'9999999999' ) )
cWide := Alltrim( Transform( (nWide*2)+2,'9999' ) )

?? cEsc + "*p" + cLeft + "x" + cTop + "Y"
?? cEsc + "*t600R"
?? cEsc + "*r0F"
?? cEsc + "*r1A"
?? cEsc + "*c15G"
?? cEsc + "*v2T"

for Cnt = 1 to ( nHeight )

  if nWide == 0
    ?? cEsc + "*b1m2W" + Chr(nWidth-1) + Chr(255)
    Else
      ?? cEsc + "*b1m" + cWide + "W" + Repl( Chr(254) + Chr(255),nWide ) + ;
                                       Chr( nReste-1 ) + Chr(255)
  End

Next

?? cEsc + "*rB"
?? cEsc + "*v0T"

@ 14,col()+2 say 'Completed'
Tone( 3000,1 )
SetColor( "R+/W,R+/W" )
@ 16,7 Say 'Hit [Esc] to continue...'
Clear TypeaHead
While Inkey(0) <> 27
End
RestScreen( 12,5,16,75,OldScr )
SetColor( OldCol )
SetCursor( OldCur )

Set( _SET_CONSOLE,OldCon )

Return( 0 )
*******************************************************************************
/* Fonction HpHLine()  Impression d'une ligne horizontale.
   Cette fonction retourne toujours NIL.
   Les paramtres sont:
        nLeft    N  Coordon horizontale du point gauche (en pouce)
        nTop     N  Coordon verticale du point gauche (en pouce)
        nWidth   N  Longueur de la ligne (en pouce)
        nHeight  N  Epaisseur de la ligne (en points, 1  8)
*/

Func HpHLine( nLeft,nTop,nLength,nThick )

Local cEsc,cTop,cLeft,nV,nPlein,nParti,cBin,cBoite,OldCon,OldScr,OldCol,OldCur

if !Set( _SET_PRINTER )
  Return( -1 )
End
OldCon := Set( _SET_CONSOLE,.f. )
OldScr := SaveScreen( 12,5,16,75 )
OldCol := SetColor( "N/W,N/W" )
OldCur := SetCursor( 0 )
cBoite := '͸Գ '
@ 12,5,16,75 Box cBoite
SetColor( "W+/B,W+/B" )
@ 12,5 Say Space(71)
@ 12,7 Say 'HpHLine() by ACTI-$OFT(r)     FOR DEMO ONLY'
SetColor( "N/W,N/W" )
@ 14,7 say 'Printing horizontal line...'

cEsc    := Chr(27)
cBin    := '00000000'

nTop    := nTop - 0.50
nTop    := if( nTop < 0,0,nTop )
nTop    := Int( nTop * 300 )
nLeft   := nLeft - 0.25
nLeft   := if( nLeft < 0,0,nLeft )
nLeft   := Int( nLeft * 300 )
nThick  := if( nThick==NIL,1,nThick )
nThick  := if( nThick<1,1,nThick )
nThick  := if( nThick>8,8,nThick )
nLength := nLength * 75
nPlein  := Int( nLength / 8 )
nParti  := nLength - ( nPlein * 8 )

cBin    := Repl( '1',nParti ) + Repl( '0',8-nParti )
cBin    := Left( cBin,8 )
nV      := CTON( cBin,2 )

cTop    := Alltrim( Transform( nTop,'9999999999' ) )
cLeft   := Alltrim( Transform( nLeft,'9999999999' ) )

?? cEsc + "*p" + cLeft + "x" + cTop + "Y"
?? cEsc + "*t75R"
?? cEsc + "*r0F"
?? cEsc + "*r1A"

for Cnt = 1 to nThick

  ?? cEsc + "*b1m4W" + Chr(nPlein-1) + chr(255) + Chr(0) + Chr(nV)

Next
?? cEsc + "*rB"

@ 14,col()+2 say 'Completed'
Tone( 3000,1 )
SetColor( "R+/W,R+/W" )
@ 16,7 Say 'Hit [Esc] to continue...'
Clear TypeaHead
While Inkey(0) <> 27
End
RestScreen( 12,5,16,75,OldScr )
SetColor( OldCol )
SetCursor( OldCur )

Set( _SET_CONSOLE,OldCon )

Return( 0 )
*******************************************************************************
/* Fonction HpVLine()  Impression d'une ligne verticale.
   Cette fonction retourne toujours NIL.
   Les paramtres sont:
        nLeft    N  Coordon horizontale du point suprieur (en pouce)
        nTop     N  Coordon verticale du point suprieur (en pouce)
        nWidth   N  Epaisseur de la ligne (en points, 1  8)Longueur de la ligne (en pouce)
        nHeight  N  Hauteur de la ligne (en pouce)
*/

Func HpVLine( nLeft,nTop,nLength,nThick )

Local cEsc,cTop,cLeft,cBin,cChar,cBoite,OldCon,OldScr,OldCol,OldCur

if !Set( _SET_PRINTER )
  Return( -1 )
End
OldCon := Set( _SET_CONSOLE,.f. )
OldScr := SaveScreen( 12,5,16,75 )
OldCol := SetColor( "N/W,N/W" )
OldCur := SetCursor( 0 )
cBoite := '͸Գ '
@ 12,5,16,75 Box cBoite
SetColor( "W+/B,W+/B" )
@ 12,5 Say Space(71)
@ 12,7 Say 'HpVLine() by ACTI-$OFT(r)     FOR DEMO ONLY'
SetColor( "N/W,N/W" )
@ 14,7 say 'Printing vertical line...'

cEsc    := Chr(27)

nTop    := nTop - 0.50
nTop    := if( nTop < 0,0,nTop )
nTop    := Int( nTop * 300 )
nLeft   := nLeft - 0.25
nLeft   := if( nLeft < 0,0,nLeft )
nLeft   := Int( nLeft * 300 )
nLength := Int( nLength * 75 )
nThick  := if( nThick==NIL,1,nThick )
nThick  := if( nThick<0,1,nThick )
nThick  := if( nThick>8,8,nThick )

cTop  := Alltrim( Transform( nTop,'9999999999' ) )
cLeft := Alltrim( Transform( nLeft,'9999999999' ) )

?? cEsc + "*p" + cLeft + "x" + cTop + "Y"
?? cEsc + "*t75R"
?? cEsc + "*r0F"
?? cEsc + "*r1A"

cBin := ''
For Cnt = 1 to 8
  if Cnt <= nThick
    cBin := cBin + '1'
    Else
      cBin := cBin + '0'
  End
Next
cChar := Chr( CTON( cBin,2 ) )

for Cnt = 1 to nLength

  ?? cEsc + "*b0m1W" + cChar

Next
?? cEsc + "*rB"

@ 14,col()+2 say 'Completed'
Tone( 3000,1 )
SetColor( "R+/W,R+/W" )
@ 16,7 Say 'Hit [Esc] to continue...'
Clear TypeaHead
While Inkey(0) <> 27
End
RestScreen( 12,5,16,75,OldScr )
SetColor( OldCol )
SetCursor( OldCur )

Set( _SET_CONSOLE,OldCon )

Return( 0 )
******************************************************************************
/* Fonction HpText()  Impression d'un texte.
   Cette fonction retourne toujours NIL.
   Les paramtres sont:
        nLeft    N  Coordon horizontale du point infrieur gauche (en pouce)
        nTop     N  Coordon verticale du point infrieur gauche (en pouce)
        cTexte   C  Texte  imprimer
        nPSize   N  Taille du caractre (en point)
        lBold    L  Caractres gras
        lUnder   L  Caractres souligns
        lItalic  L  Caractres italiques.
*/

Func HpText( nLeft,nTop,cTexte,nPSize,lBold,lUnder,lItalic )

Local cEsc,cSize,cBold,cUnder,cItalic,cTop,cLeft,cBoite,OldCon,OldScr,OldCol,OldCur

if !Set( _SET_PRINTER )
  Return( -1 )
End
OldCon := Set( _SET_CONSOLE,.f. )
OldScr := SaveScreen( 12,5,16,75 )
OldCol := SetColor( "N/W,N/W" )
OldCur := SetCursor( 0 )
cBoite := '͸Գ '
@ 12,5,16,75 Box cBoite
SetColor( "W+/B,W+/B" )
@ 12,5 Say Space(71)
@ 12,7 Say 'HpText() by ACTI-$OFT(r)     FOR DEMO ONLY'
SetColor( "N/W,N/W" )
@ 14,7 say 'Printing text...'

cEsc    := Chr(27)

lBold   := if( lBold == NIL,.f.,lBold )
lUnder  := if( lUnder == NIL,.f.,lUnder )
lItalic := if( lItalic == NIL,.f.,lItalic )
nPSize  := if( nPSize == NIL,12,nPSize )

cSize   := Strzero( nPSize,6,2 )
cBold   := if( lBold,'3','0' )
cUnder  := if( lUnder,'0D','@' )
cItalic := if( lItalic,'1','0' )

nLeft   := nLeft - .25
nLeft   := if( nLeft < 0,0,nLeft )
nLeft   := nLeft * 300
cLeft   := Alltrim( Transform( nLeft,'9999999999' ) )
nTop    := nTop - .50
nTop    := if( nTop <0,0,nTop )
nTop    := nTop * 300
cTop    := Alltrim( Transform( nTop,'9999999999' ) )

?? cEsc + '*p' + cLeft + 'x' + cTop + 'Y' + ;
   cEsc + '(10U' + ;                // Symbol Set Selection "PC-8"
   cEsc + '(s1P' + ;                // Spacing "Proportional"
   cEsc + '(s' + cSize + 'V' + ;    // Point size
   cEsc + '(s' + cItalic + 'B' + ;  // Style "Italic"
   cEsc + '(s' + cBold + 'B' + ;    // Stroke weight "Bold"
   cEsc + '(s4101T' + ;             // Primary Type Face "CG Times"
   cEsc + '&d' + cUnder + ;         // Underlining
   cTexte

@ 14,col()+2 say 'Completed'
Tone( 3000,1 )
SetColor( "R+/W,R+/W" )
@ 16,7 Say 'Hit [Esc] to continue...'
Clear TypeaHead
While Inkey(0) <> 27
End
RestScreen( 12,5,16,75,OldScr )
SetColor( OldCol )
SetCursor( OldCur )

Set( _SET_CONSOLE,OldCon )

Return( 0 )
*******************************************************************************
Func HpLChart( nXCoord,nYCoord,nWidth,nHeight,cTitle,cXName,cYName,aData,;
               lPrintValues )

Local cEsc,Cnt,nXMax,nYMax,nXPos,nYPos

if !Set( _SET_PRINTER )
  Return( -1 )
End

if nWidth <= 0 .or. nHeight <= 0 .or. Len( aData ) == 0
  Return( -1 )
End

if aData[Len(aData)][1] == 0
  Return( -1 )
End

OldCon := Set( _SET_CONSOLE,.f. )
OldScr := SaveScreen( 12,5,16,75 )
OldCol := SetColor( "N/W,N/W" )
OldCur := SetCursor( 0 )
cBoite := '͸Գ '
@ 12,5,16,75 Box cBoite
SetColor( "W+/B,W+/B" )
@ 12,5 Say Space(71)
@ 12,7 Say 'HpLChart() by ACTI-$OFT(r)     FOR DEMO ONLY'
SetColor( "N/W,N/W" )
@ 14,7 say 'Printing Line chart...'

cEsc    := Chr(27)

nXCoord := nXCoord - .25
nXCoord := if( nXCoord < 0,0,nXCoord )

nYCoord := nYCoord - .50
nYCoord := if( nYCoord < 0,0,nYCoord )

?? cEsc + "*p" + Ltrim( Transform( (nXCoord+.1875)*300,'999999' ) ) + 'x' + ;
                 Ltrim( Transform( (nYCoord-.50)*300,'999999' ) ) + 'Y' + ;
   cEsc + "&a090P" + ;              // 90 degr orientation
   cEsc + '(10U' + ;                // Symbol Set Selection "PC-8"
   cEsc + '(s1P' + ;                // Spacing "Proportional"
   cEsc + '(s10V' + ;               // Point size
   cYName

?? cEsc + "&a000P" + ;              // 0 degr orientation
   cEsc + "*p" + Ltrim( Transform( (nXCoord+.50)*300,'999999' ) ) + 'x' + ;
                 Ltrim( Transform( (nYCoord-.0625)*300,'999999' ) ) + 'Y' + ;
   cEsc + '(10U' + ;                // Symbol Set Selection "PC-8"
   cEsc + '(s1P' + ;                // Spacing "Proportional"
   cEsc + '(s10V' + ;               // Point size
   cXName

?? cEsc + "*p" + Ltrim( Transform( (nXCoord+.50)*300,'999999' ) ) + 'x' + ;
                 Ltrim( Transform( (nYCoord-(nHeight-.25))*300,'999999' ) ) + 'Y' + ;
   cEsc + '(10U' + ;                // Symbol Set Selection "PC-8"
   cEsc + '(s1P' + ;                // Spacing "Proportional"
   cEsc + '(s12V' + ;               // Point size
   cTitle

nXCoord := ( nXCoord * 1016 ) + 254
nXCoord := if( nXCoord < 0,0,nXCoord )
nYCoord := 10160 - ( nYCoord * 1016 ) + 254
nYCoord := if( nYCoord < 0,0,nYCoord )

nWidth  := ( nWidth - .50 ) * 1016
nHeight := ( nHeight - .75 ) * 1016

?? cEsc + "%1B"                                    // Enter HP-GL/2
?? "DF;IN;SP1;"                                    // Default and initialize
?? "PW2;"
?? "PA" + Ltrim( Transform( nXCoord+nWidth,'999999' ) ) + ',' + ;
          LTrim( Transform( nYCoord,'999999' ) ) + ';'
?? "PD" + Ltrim( Transform( nXCoord,'999999' ) ) + ',' + ;
          Ltrim( Transform( nYCoord,'999999' ) ) + ',' + ;
          Ltrim( Transform( nXCoord,'999999' ) ) + ',' + ;
          Ltrim( Transform( nYCoord+nHeight,'999999' ) ) + ';'
?? "PU;PW1;"
?? cEsc + "%0A"

if !aData[1][1] == 1 .or. ;
   !aData[1][2] == 2
  aAdd( aData,{0,0} )
  aIns( aData,1 )
  aFill( aData,{0,0},1,1 )
End

nXMax := nYMax := 0
For Cnt = 1 to Len( aData )
  nXMax := if( aData[Cnt][1] > nXMax,aData[Cnt][1],nXMax )
  nYMax := if( aData[Cnt][2] > nYMax,aData[Cnt][2],nYMax )
Next
?? cEsc + "%1B"                                    // Enter HP-GL/2
?? "DF;IN;SP1;"                                    // Default and initialize
?? "PD" + Ltrim( Transform( nXCoord,'999999' ) ) + ',' + ;
          LTrim( Transform( nYCoord,'999999' ) ) + ';'

For Cnt = 2 to Len( aData )
  nXPos := nXCoord + ( ( aData[Cnt][1] * nWidth ) / nXMax )
  if !nYMax == 0
    nYPos := nYCoord + ( ( aData[Cnt][2] * nHeight ) / nYMax )
    Else
      nYPos := nYCoord
  End
  ?? "PA" + ;
     LTrim( Transform( nXPos,'999999' ) ) + ',' + ;
     LTrim( Transform( nYPos,'999999' ) ) + ';'
  ?? "FT1;WG-20,0,360;"
Next
?? "PU;"
?? cEsc + "%0A"

@ 14,col()+2 say 'Completed'
Tone( 3000,1 )
SetColor( "R+/W,R+/W" )
@ 16,7 Say 'Hit [Esc] to continue...'
Clear TypeaHead
While Inkey(0) <> 27
End
RestScreen( 12,5,16,75,OldScr )
SetColor( OldCol )
SetCursor( OldCur )

Set( _SET_CONSOLE,OldCon )

Return( 0 )
*******************************************************************************
